%%
%% OzInstall Toplevel Window
%%

functor

import
   
   QTk at 'http://www.info.ucl.ac.be/people/ned/qtk/QTk.ozf'

   ProgressDisplay
   MessageDisplay
   About
   
export
   'class': Toplevel
   
define
   
   DefaultTitle = "Mozart Installer (alpha)"

   BG = grey
   
   %%
   %% Progress Display
   %%

   ProgressColors = colors(default:c(235 245 255))

   local
      LabelBG = ProgressColors.default
   in
      fun {MkProgressDisplay Stages}
         Handle#Desc = {ProgressDisplay.mkDesc Stages ProgressColors}
      in
         Handle # td(glue:nws bg:BG
                     label(glue:nwe bg:LabelBG text:"Progress")
                     {AdjoinAt Desc glue nw})
      end
   end
   
   %%
   %% Program Messages Protocol 
   %%
   /*
   fun {MkItem N Txt H}
      td(glue:nwse handle:H
         label(text:{LastWord Txt})
         text(glue:nwse feature:text bg:lightgrey
              tdscrollbar:true init:"" state:disabled))
   end
   */
   fun {MkMessageDisplay Stages}
      Handle#Desc = {MessageDisplay.mkDesc}
   in
      Handle # td(glue:nwse
                  lr(glue:we
                     label(glue:we text:"Progress Messages")
                     menubutton(text:"?" menu:menu(About.menuEntry))
                    )
                  Desc)
   end
   
   %%
   %% Cache selection
   %%   
   fun {MkCacheSelection Caches}
      Handle
      Desc = lr(glue:we handle:Handle
                label(glue:w text:"Destination:")
                tdspace(glue:w)
                radiobutton(
                   glue:w group:cache feature:'local' text:"Local cache"
                   action: proc{$}{Handle.entry set(Caches.'local')} end)
                radiobutton(
                   glue:w group:cache feature:'global' text:"Global cache"
                   action: proc{$} {Handle.entry set(Caches.'global')} end)
                tdspace(glue:w)
                label(glue:e text:"selected:")
                entry(glue:we feature:entry init:"")
               ) 
   in
      Handle#Desc
   end

   %%
   %% Toplevel Window
   %%
   fun {MkToplevel Title Stages Caches}
      ProgressHandle # ProgressDesc = {MkProgressDisplay Stages}
      MessageHandle  # MessageDesc  = {MkMessageDisplay Stages}
      CacheHandle    # CacheDesc    = {MkCacheSelection Caches}

      %% "Install" button
      InstallHandle CacheSelection
      InstallButton = button(
                         /*glue:w*/ text:"Install" handle:InstallHandle
                         action: proc {$}
                                    {InstallHandle set(state:disabled)}
                                    {CacheHandle.'local' set(state:disabled)}
                                    {CacheHandle.'global' set(state:disabled)}
                                    {CacheHandle.entry set(state:disabled)}
                                    CacheSelection = {CacheHandle.entry get($)}
                                 end)
      
      %% "Exit" button
      ExitHandle ExitSignal
      ExitButton = button(/*glue:e*/ text:"Exit" handle:ExitHandle
                          action:proc {$} ExitSignal=unit end)
      %% main
      Hspace = lrspace(glue:we bg:BG)
      Vspace = tdspace(glue:ns bg:BG)
      Hline  = lrline(glue:we bg:BG)
      %%Vline  = tdline(glue:ns bg:BG)
      
      Desc = td(bg:BG
                Hspace
                lr(glue:nwe bg:BG
                   Vspace
                   ProgressDesc
                   Vspace
                   td(glue:ne
                      MessageDesc
                      Hspace
                      CacheDesc
                      Hspace Hline Hspace
                      lr(glue:we bg:BG InstallButton ExitButton)
                     )
                   Vspace
                  )
                Hspace
               )
      
      {{QTk.build {Adjoin unit(title:Title) Desc}} show}
      ProgressObj = {New ProgressDisplay.'class' init(ProgressHandle
                                                      colors:ProgressColors)}
      MessageObj = {New MessageDisplay.'class' init(MessageHandle 1)}
   in
      {CacheHandle.(Caches.default) set(true)}
      {CacheHandle.entry set(Caches.(Caches.default))}
      {ExitHandle set(state:disabled)}
      
      handles(progress:        ProgressObj
              messages:        MessageObj
              cache:           CacheHandle
              cacheSelection:  CacheSelection
              exit:            ExitHandle
              exitSignal:      ExitSignal)
   end

   class Toplevel
      feat
         progress
         messages
         cache
         cacheSelection
         exit
         exitSignal
         
      meth init(title:  Title <= DefaultTitle
                stages: Stages
                caches: Caches)
         Handles = {MkToplevel Title Stages Caches} in
         self.progress        = Handles.progress
         self.messages        = Handles.messages
         self.cache           = Handles.cache
         self.cacheSelection  = Handles.cacheSelection
         self.exit            = Handles.exit
         self.exitSignal      = Handles.exitSignal
         {self bindClick(1)}
      end
      meth Select(N)
         {self.messages select(N)}
      end
      meth select(N)
         if {self.messages exists(N $)}==false then
            {self add(N)}
         end
         {self Select(N)}
      end
      meth add(N)
         {self.messages create(N)}
         {self bindClick(N)}
      end
      meth bindClick(N)
         if {self.progress exists(N $)} then
            proc {Action} {self Select(N)} end in
            {self.progress send(N bind(event:"<1>" action:Action))}
         end
      end
      meth enter(...)=M
         {self.progress M}
         {self select({self.progress current($)})}
      end
      meth leave(...)=M {self.progress M} end
      meth error(...)=M {self.progress M} end
      meth printInfo(Text)
         {self.messages printInfo(Text)}
      end
      meth showInfo(Text)  {self printInfo(Text#"\n")} end
      meth showError(Text) {self printInfo(Text#"\n")} end
      meth cacheSelection($)
         self.cacheSelection
      end
      meth exitSignal($)
         self.exitSignal
      end
      meth enableExit  {self.exit set(state:normal)}   end
      meth disableExit {self.exit set(state:disabled)} end
   end    
end



